﻿using System.Security.Cryptography;
using System.Text;
using System.Web.UI.WebControls;
using Lephone.Util;
using Lephone.Util.Text;
using Lephone.Web;

namespace VisualWiki.Models.Helpers
{
    public static class CommonHelper
    {
        public static void SetLink(HyperLink link, string pageName, string linkPageName, string title, string path)
        {
            if (pageName.ToLower() == linkPageName.ToLower())
            {
                link.Enabled = false;
            }
            else
            {
                link.Enabled = true;
                link.NavigateUrl = new UrlBuilder(linkPageName).Add(Const.PathName, path).Add(Const.TitleName, title).ToString();
            }
        }

        public static string GenerateShowText(string text, string path)
        {
            return ProcessSymbol(text, "[[", "]]",
                title =>
                {
                    var url = new UrlBuilder("Show.aspx");
                    if (!string.IsNullOrEmpty(path))
                    {
                        url.Add(Const.PathName, path);
                    }
                    url.Add(Const.TitleName, RemoveHtmlTag(title));
                    var link = HtmlBuilder.New.a(url.ToString()).include(title).end;
                    return link.ToString();
                });
        }

        public static string RemoveHtmlTag(string text)
        {
            return ProcessSymbol(text, "<", ">", null);
        }

        public static string ProcessSymbol(string text, string left, string right, 
            CallbackHandler<string, string> callback)
        {
            var ret = new StringBuilder();
            int last = 0;
            while (true)
            {
                int m = text.IndexOf(left, last);
                if (m >= last)
                {
                    int n = text.IndexOf(right, m);
                    if (n > m)
                    {
                        ret.Append(text.Substring(last, m - last));
                        if(callback != null)
                        {
                            var inner = text.Substring(m + left.Length, n - m - left.Length);
                            ret.Append(callback(inner));
                        }
                        last = n + right.Length;
                        continue;
                    }
                }
                ret.Append(text.Substring(last));
                break;
            }
            return ret.ToString();
        }

        public static string GetNavigationBar(string title, string path)
        {
            var buildUrls = new StringBuilder();
            string buildPath = "";
            buildUrls.Append(GetNavigationItem(Const.HomeName, null)).Append(Const.MenuSpliter);
            if (title != null)
            {
                if (path != null)
                {
                    string[] ss = path.Split(Const.ParameterSpliter);
                    foreach (var s in ss)
                    {
                        buildUrls.Append(GetNavigationItemById(s, buildPath));
                        buildUrls.Append(Const.MenuSpliter);
                        if (buildPath == "")
                        {
                            buildPath = s;
                        }
                        else
                        {
                            buildPath += Const.ParameterSpliter + s;
                        }
                    }
                }
                if (title != Const.HomeName)
                {
                    buildUrls.Append(GetNavigationItem(title, path)).Append(Const.MenuSpliter);
                }
            }
            buildUrls.Length -= Const.MenuSpliter.Length;
            return buildUrls.ToString();
        }

        private static string GetNavigationItemById(string sid, string buildPath)
        {
            var id = long.Parse(sid);
            var article = Article.FindById(id);
            return GetNavigationItem(article.Title, buildPath);
        }

        private static string GetNavigationItem(string title, string path)
        {
            var html = HtmlBuilder.New;
            var url = new UrlBuilder("Show.aspx");
            if (!string.IsNullOrEmpty(path))
            {
                url.Add(Const.PathName, path);
            }
            url.Add(Const.TitleName, title);
            return html.a(url.ToString()).text(title).end.ToString();
        }

        private static readonly SHA512 Hash = SHA512.Create();

        public static string GetHashedPassword(string password)
        {
            var bytes = Hash.ComputeHash(Encoding.UTF8.GetBytes(password));
            var ret = Base32StringCoding.Decode(bytes);
            return ret;
        }
    }
}
