﻿using Lephone.Data;
using NUnit.Framework;
using VisualWiki.Models;
using VisualWiki.Models.Helpers;

namespace VisualWiki.UnitTests
{
    [TestFixture]
    public class TestHelper : DbTestBase
    {
        [Test]
        public void TestTestGenerateShowTest1()
        {
            const string text = "abc [[bb]] xyz";
            var act = CommonHelper.GenerateShowText(text, null);
            Assert.AreEqual("abc <a href=\"Show.aspx?title=bb\">bb</a> xyz", act);
        }

        [Test]
        public void TestTestGenerateShowTest2()
        {
            const string text = "abc [[bb xyz";
            var act = CommonHelper.GenerateShowText(text, null);
            Assert.AreEqual("abc [[bb xyz", act);
        }

        [Test]
        public void TestTestGenerateShowTest3()
        {
            const string text = "abc bb]] xyz";
            var act = CommonHelper.GenerateShowText(text, null);
            Assert.AreEqual("abc bb]] xyz", act);
        }

        [Test]
        public void TestTestGenerateShowTest4()
        {
            const string text = "[[bb]]";
            var act = CommonHelper.GenerateShowText(text, null);
            Assert.AreEqual("<a href=\"Show.aspx?title=bb\">bb</a>", act);
        }

        [Test]
        public void TestTestGenerateShowTest5()
        {
            const string text = "[[x<b>y</b>z]]";
            var act = CommonHelper.GenerateShowText(text, null);
            Assert.AreEqual("<a href=\"Show.aspx?title=xyz\">x<b>y</b>z</a>", act);
        }

        [Test]
        public void TestTestGenerateShowTest6()
        {
            const string text = "[[bb]] aa [[cc]]";
            var act = CommonHelper.GenerateShowText(text, null);
            Assert.AreEqual("<a href=\"Show.aspx?title=bb\">bb</a> aa <a href=\"Show.aspx?title=cc\">cc</a>", act);
        }

        [Test]
        public void TestTestGenerateShowTestWithPath()
        {
            var text = "abc [[bb]] xyz";
            var act = CommonHelper.GenerateShowText(text, "1_2");
            Assert.AreEqual("abc <a href=\"Show.aspx?path=1_2&title=bb\">bb</a> xyz", act);
        }

        [Test]
        public void TestProcessSymbol()
        {
            var result = CommonHelper.ProcessSymbol("[[abc)", "[[", ")", text => text);
            Assert.AreEqual("abc", result);

            result = CommonHelper.ProcessSymbol("1111[[[[abc)2222", "[[[[", ")", text => text);
            Assert.AreEqual("1111abc2222", result);

            result = CommonHelper.ProcessSymbol("1111(abc]]]]2222", "(", "]]]]", text => text);
            Assert.AreEqual("1111abc2222", result);
        }

        [Test]
        public void TestGetNavigationBar()
        {
            DbEntry.Context.DropAndCreate(typeof(Article));
            Article.New.Init("Home", "abc [[Hello]] xyz").Save();
            Article.New.Init("Hello", "abc [[Next]] xyz").Save();
            Article.New.Init("Next", "abc [[Last]] xyz").Save();

            var result = CommonHelper.GetNavigationBar("Home", null);
            Assert.AreEqual(@"<a href=""Show.aspx?title=Home"">Home</a>", result);

            result = CommonHelper.GetNavigationBar("Hello", null);
            Assert.AreEqual(@"<a href=""Show.aspx?title=Home"">Home</a> &gt; <a href=""Show.aspx?title=Hello"">Hello</a>", result);

            result = CommonHelper.GetNavigationBar("Next", "2");
            Assert.AreEqual(@"<a href=""Show.aspx?title=Home"">Home</a> &gt; <a href=""Show.aspx?title=Hello"">Hello</a> &gt; <a href=""Show.aspx?path=2&title=Next"">Next</a>", result);

            result = CommonHelper.GetNavigationBar("Last", "2_3");
            Assert.AreEqual(@"<a href=""Show.aspx?title=Home"">Home</a> &gt; <a href=""Show.aspx?title=Hello"">Hello</a> &gt; <a href=""Show.aspx?path=2&title=Next"">Next</a> &gt; <a href=""Show.aspx?path=2_3&title=Last"">Last</a>", result);
        }
    }
}
